package hr.genetics.models;

import java.util.ArrayList;
import java.util.List;

import hr.geometrics.Line;
import hr.geometrics.Point;
import hr.geometrics.Vector;

public class Simulator {

	private static int maxTries = 1;
	private static int distanceLimit = 10000;
	
	private SimulationModel model;
	private Path path;
	private List<Point> startPoints;
	
	private int score;
		
	public Simulator(GenericVechileModel vechile, Path path) {
		
		this.model = new SimulationModel(vechile);
		this.path = path;
		
		vechile.setReality(this.model);
		vechile.initialiseSensors();
		
		this.score = 0;
		
		this.startPoints = new ArrayList<Point>();
		
		this.model.setEnvironment(this.path);
		
		this.startSimulations();
				
	}
	
	public void startSimulations() {
		
		Point startPosition = null;
		for (int tryNo = 0; tryNo < maxTries; ++tryNo) {
			startPosition = this.putVechileInEnvironment();
			this.simulate();
			this.addScore(this.calculateFitness());
			this.startPoints.add(startPosition);
		}
		
	}
	
	private Point putVechileInEnvironment() {
		
		Line startLine = this.path.getStartLine();
		
		Point position = new Point( 
				new Double((startLine.getX1() + startLine.getX2())/2).intValue(),
				startLine.getY1()
		);
		
		this.model.setSpeed(new Vector(0, -1));
		this.model.changePosition(position);
		
		return position;
		
	}
	
	private void simulate() {
				
		Point currentPosition = this.model.getPosition();
		// setting differences to zero
		this.model.changePosition(currentPosition);
		
		Vector currentSpeed = null;
		
		while (!this.model.isCrashed() && !this.model.isFinished()
				&& this.model.getDrivenDistance() < distanceLimit
		) {
			currentPosition = this.model.getPosition();
			currentSpeed = this.model.getNewSpeed();
			this.model.changePosition(
					new Point(
							currentPosition.getX() + currentSpeed.getX(),
							currentPosition.getY() + currentSpeed.getY()
					)
			);
			
		}
		
	}
	
	private int calculateFitness() {
		
		int fitness = 0;
		
		fitness += (this.model.passedCheckpoints())*200;
				
		if (this.model.isCrashed())
			fitness -= 1000;
		
		if (this.getSimulationModel().getPosition().getY() < 1) 
			fitness -= 1000;
		
		if (this.model.isFinished()) {
			fitness += 2000;
			fitness -= this.model.getDrivenDistance();
		}
				
		fitness += 5*(300 - new Double(this.model.getPosition().getY()).intValue());
		
		return fitness;
		
	}
	
	public int getScore() {
		
		return this.score;
		
	}
	
	private void addScore(int score) {
		this.score += score;
	}
	
	public SimulationModel getSimulationModel() {
		return this.model;
	}
	
	public List<SimulationState> getReplay() {
		return this.model.replay;
	}
	
}
