function PathViewer(path, canvasContainerSelector) {
	
	if (!(path instanceof Path)) {
		console.log("PathViewer must get instance of Path as first argument.");
		return null;
	}
	
	if (!this.path)
		this.path = path;
	
	if (!this.canvasContainerSelector)
		this.canvasContainerSelector = canvasContainerSelector;
	
	this.prepareCanvas();
	this.instantiateOCanvas();
	
	return this;
	
}

	PathViewer.prototype.canvasContainerSelector = null;
	PathViewer.prototype.canvasDom = null;
	PathViewer.prototype.ocanvas = null;
	
	PathViewer.prototype.startZone = null;
	PathViewer.prototype.drivingZone = null;
	PathViewer.prototype.endZone = null;
	
	PathViewer.prototype.lastLinesSize = 0;
	PathViewer.prototype.lastCheckpointsSize = 0;
	PathViewer.prototype.startLineCreated = false;
	
	PathViewer.prototype.options = {
		canvasClass: "path_builder",
		paddingVertical: 40,
	};
	
	PathViewer.prototype.prepareCanvas = function() {
		
		this.canvasDom = $('canvas');
		$(this.canvasContainerSelector+' canvas').remove();
		
		this.canvasDom = $(document.createElement('canvas'));
		
		this.canvasDom.addClass(this.options.canvasClass)
						.attr('width', this.path.getWidth())
						.attr('height', this.path.getHeight())
						.width(this.path.getWidth())
						.height(this.path.getHeight())
						.attr('id', 'path').
						appendTo('#canvas_controls');
								
	};
	
	PathViewer.prototype.instantiateOCanvas = function() {
		
		this.ocanvas = oCanvas.create({
			canvas: '#path',
			fps: 30
		});
		
	};
	
	PathViewer.prototype.drawBuildingZones = function() {
		
		var startZone = this.ocanvas.display.rectangle({
			x: 0,
			y: this.path.getHeight() - this.options.paddingVertical,
			width: this.path.getWidth(),
			height: this.options.paddingVertical,
			fill: "#bcffc1",
			opacity: 0.8
		});
		
		var endZone = this.ocanvas.display.rectangle({
			x: 0,
			y: 0,
			width: this.path.getWidth(),
			height: this.options.paddingVertical,
			fill: "#ffa4a4",
			opacity: 0.8
		});
		
		var drivingZone = this.ocanvas.display.rectangle({
			x: 0,
			y: this.options.paddingVertical,
			width: this.path.getWidth(),
			height: this.path.getHeight() - 2*this.options.paddingVertical,
			fill: "#f1f1f1",
			opacity: 0.8
		});
		
		
		this.ocanvas.addChild(startZone);
		this.ocanvas.addChild(endZone);
		this.ocanvas.addChild(drivingZone);
		
		this.startZone = startZone;
		this.drivingZone = drivingZone;
		this.endZone = endZone;
		
	};
	
	PathViewer.prototype.getStartZone = function() {
		return this.startZone;
	};
	
	PathViewer.prototype.getDrivingZone = function() {
		return this.drivingZone;
	};
	
	PathViewer.prototype.getEndZone = function() {
		return this.endZone;
	};
	
	PathViewer.prototype.getCanvas = function() {
		return this.ocanvas;
	};
	
	PathViewer.prototype.updateView = function() {

		// update lines
		if (this.lastLinesSize < this.path.lines.length) {
			for (var i = this.lastLinesSize; i < this.path.lines.length; ++i) {
				var line = this.path.lines[i];
				this.ocanvas.addChild(this.ocanvas.display.line({
					start: { x: line.x1, y: line.y1 },
					end: { x: line.x2, y: line.y2 },
					stroke: "2px #0aa",
					cap: "round"
				}));
				
				this.ocanvas.addChild(this.ocanvas.display.ellipse({
					x: line.x2,
					y: line.y2,
					radius: 3,
					stroke: "1px #0aa"
				}));
			}
			this.lastLinesSize = this.path.lines.length;
		}
		
		// update checkpoints
		if (this.lastCheckpointsSize < this.path.checkpoints.length) {
			for (var i = this.lastCheckpointsSize; i < this.path.checkpoints.length; ++i) {
				var checkpoint = this.path.checkpoints[i];
				this.ocanvas.addChild(this.ocanvas.display.ellipse({
					x: checkpoint.point.x,
					y: checkpoint.point.y,
					radius: 5,
					fill: "#ffb6bb",
					stroke: "2px #ff5c5c"
				}));
			}
			
			this.lastCheckpointsSize = this.path.checkpoints.length;
		}
		
		// update startline
		if (!this.startLineCreated && this.path.startLine != null) {
			
			var line = this.path.startLine;
			
			this.ocanvas.addChild(this.ocanvas.display.line({
				start: { x: line.x1, y: line.y1 },
				end: { x: line.x2, y: line.y2 },
				stroke: "2px #666",
				cap: "round"
			}));
			
			this.ocanvas.addChild(this.ocanvas.display.ellipse({
				x: line.x1,
				y: line.y1,
				radius: 3,
				stroke: "1px #666"
			}));
			
			this.ocanvas.addChild(this.ocanvas.display.ellipse({
				x: line.x2,
				y: line.y2,
				radius: 3,
				stroke: "1px #666"
			}));
			
			this.startLineCreated = true;
		}
				
	};
	