(function( window, undefined ) {

	window.$storage = {
			
		storage: {},
				
		setItem: function(name, value) {
			this.storage[name] = value;
		},
			
		getItem: function(name) {
			return this.storage[name];
		}
				
	};
		

	window.Storage = {
		
		get: function(name) {
			return $storage.getItem(name);
		},
		
		update: function(name, value) {
			$storage.setItem(name, value);
		},
		
		getPathSets: function() {

			var pathSets = this.get('PathSets');
			if (!pathSets) {
				pathSets = new Array();
			}
			return pathSets;

		},
		
		getPathSet: function(id) {
			
			var pathSets = this.get('PathSets');
			
			for (var i = 0; i < pathSets.length; ++i) {
				if (pathSets[i].id == id) {
					return pathSets[i];
				}
			}
			
			return null;
			
		},
		
		addPathSet: function(pathSet) {

			var pathSets = this.getPathSets();
			
			pathSets.push(pathSet);
			
			this.update("PathSets", pathSets);

		},
		
		removePathSet: function(setId) {
			
			var pathSets = this.getPathSets();
			
			var newPathSets = new Array();
			for (var i = 0; i < pathSets.length; ++i) {
				if (pathSets[i].id != setId) {
					newPathSets.push(pathSets[i]);
				}
			}
			
			this.update("PathSets", newPathSets);
			
		},
		
		getPaths: function() {
			
			var paths = this.get("Paths");
			
			if (!paths) {
				paths = new Array();
			}
			
			return paths;
			
		},
		
		addPath: function(path) {
						
			var paths = this.getPaths();
			paths.push(path);
			
			this.update("Paths", paths);
		},
		
		removePath: function(pathId) {
			
			var paths = this.getPaths();
			
			var newPaths = new Array();
			for (var i = 0; i < paths.length; ++i) {
				if (paths[i].id != pathId) {
					newPaths.push(paths[i]);
				}
			}
			
			$.ajax({
				type: 'DELETE',
				url: "/genetics/Path?pathID="+pathId,
				success: function(data){
					console.log(data);
				}
			});
			
			this.update("Paths", newPaths);
			
		},
		
		getPath: function (pathId) {
			
			var paths = this.getPaths();
			var path = null;
						
			for (var i = 0; i < paths.length; ++i) {
				if (paths[i].id == pathId) {
					path = paths[i];
				}
			}
			
			return path;
			
		},
		
		updatePath: function(path) {
			
			var paths = this.getPaths();
			
			for (var i = 0; i < paths.length; ++i) {
				if (paths[i].id == path.id) {
					paths[i] = path;
				}
			}
			
			$.ajax({
				type: 'POST',
				url: "/genetics/Path",
				data: {
					path: this.getPathExportJSON(path),
				},
				success: function(data){}
			});
			
			this.update("Paths", paths);
			
		},
		
		getExportJSON: function() {
			
			var data;
			var output = "";
			
			data = this.getPaths();
			for (var path in data) {
				output += this.getPathExportJSON(data[path]);
			}
			
			return output;
			
		},
		
		getPathExportJSON: function(data) {
			
			var output = "PATH "+ data.name +"\n";
			
			var startLine = data.startLine;
			output += "START "+startLine.x1+" "+startLine.y1+" "+startLine.x2+" "+startLine.y2+"\n";
			
			output += "NAME "+data.name+"\n";
			output += "ID "+data.id+"\n";
			output += "WIDTH "+data.width+"\n";
			output += "HEIGHT "+data.height+"\n";
			
			for (var line in data.lines) {
				output += "LINE "+data.lines[line].x1 + " " + data.lines[line].y1
						+ " " + data.lines[line].x2 + " " + data.lines[line].y2 + "\n";
			}
			
			for (var c in data.checkpoints) {
				output += "CHECKPOINT "+data.checkpoints[c].point.x+" "
							+data.checkpoints[c].point.y
						+" "+data.checkpoints[c].weight+"\n";
			}
			
			return output;
			
		},
		
		importFromJSON: function(importText) {
			
			var data = window.sharedPaths;
			var currPath;
			for (var key in data) {
				
				currPath = new Path();
				
				currPath.id = data[key].id;
				currPath.name = data[key].name;
				currPath.width = data[key].width;
				currPath.height = data[key].height;
				
				var currLine;
				for (var i in data[key].lines) {
					currLine = data[key].lines[i];
					currPath.addLine(currLine.x1, currLine.y1, currLine.x2, currLine.y2);
				}
				
				var currPoint;
				for (var i in data[key].checkpoints) {
					currPoint = data[key].checkpoints[i];
					currPath.addCheckpoint(currPoint.point.x, currPoint.point.y);
				}
				
				currPath.sides = 2;
				
				currPath.setStartLine(data[key].startLine.x1,
						data[key].startLine.y1,
						data[key].startLine.x2,
						data[key].startLine.y2);
				
				this.addPath(currPath);
			}
									
		}
		
	};
	
	if (typeof(window) !== 'undefined') {
		window.Storage = Storage;
	}
	
})(window);
	